*
* Le manoir de la crapule
* Fichier : Froggy.System (le loader)
*
* (c) 1987, Jean-Louis Le Breton, Froggy Software
* (c) 2017, Antoine Vignau & Olivier Zardini, Brutal Deluxe Software
*

	mx	%11
	org	$bc00
	lst	off
	dsk	Froggy.System

*------------------------------------------------------------------------------
* Equates & friends
*------------------------------------------------------------------------------

*--- Des equates

engineVERSION	=	$13	; this is version 1.0 (1.2 would be $12)

sizeCONDITIONS	=	90	; le moteur gère 90 conditions au maximum

false	=	$00	; valeur FAUX
true	=	$80	; valeur VRAI

dpFROM	=	$f6	; $F6 - global zero page usage
dpTO	=	dpFROM+2	; $F8
dpDATA	=	dpTO+2	; $FA
dpWORK	=	dpDATA+4	; $FE

*--- Memory usage / ProDOS equates

ptrPREFIX	=	$0280	; address of the ProDOS prefix
ptrIMAGE	=	$2000	; a text/pic image is visible at $2000
loadBUFFER	=	$b000	; ProDOS read buffer
proBUFFER	=	$b800	; ProDOS open buffer
PRODOS	=	$bf00	; The MLI entry point
MACHID	=	$bf98	; The machine ID calculated by ProDOS

mask128K	=	%00110000	; 01: 48K, 10: 64K, 11: 128K+
mask80COL	=	%00000010	; 0: 40-col, 1: 80-col

*--- Game data equates

data48K	=	%00010000	; 01: 48K, 10: 64K, 11: 128K+
data64K	=	%00100000	; 01: 48K, 10: 64K, 11: 128K+
data128K	=	%00110000	; 01: 48K, 10: 64K, 11: 128K+

text40COL	=	%00000000	; 0: 40-col, 1: 80-col
text80COL	=	%00000010	; 0: 40-col, 1: 80-col

picNOPICS	=	%00000000	; no picture, text only game
picTEXT40	=	%00000001	; 40x24
picTEXT80	=	%00000010	; 80x24
picGR	=	%00000011	; 40x48
picDGR	=	%00000100	; 80x48
picHGR	=	%00000101	; 140x192
picHGRMONO	=	%00000110	; 280x192
picDHGR	=	%00000111	; 140x192
picDHGRMONO	=	%00001000	; 560x192
picSHR320	=	%00001001	; 320x200
picSHR640	=	%00001010	; 640x200
picUNSUPPORTED	=	%00001011	; unsupported

compNONE	=	%00000000	; picture is not compressed
compLZ4	=	%00000001	; picture uses LZ4
compUNSUPPORTED =	%00000010	; unsupported

*--- Game pointers

leJEU	=	$0800
initGAME	=	leJEU+3
initTEXT	=	initGAME+3
chargeIMAGE	=	initTEXT+3	; Image number is numDESSIN
chargeIMAGEALT	=	chargeIMAGE+3	; You provide the image number
switchVIDEOTER	=	chargeIMAGEALT+3
setDATA	=	switchVIDEOTER+3
setPOINTER	=	setDATA+3
setHOOK	=	setPOINTER+3	; X=high ptr, Y=low ptr to a table, read carefully
getHOOKTABLE	=	setHOOK+3
	
*--- The softswitches

KBD	=	$c000
CLR80COL	=	$c000
SET80COL	=	$c001
CLR80VID	=	$c00c
SET80VID	=	$c00d
KBDSTROBE	=	$c010
MONOCOLOR	=	$c021
NEWVIDEO	=	$c029
TXTCLR	=	$c050
TXTSET	=	$c051
MIXCLR	=	$c052
MIXSET	=	$c053
TXTPAGE1	=	$c054
TXTPAGE2	=	$c055
LORES	=	$c056
HIRES	=	$c057
CLRAN	=	$c05d
SETAN3	=	$c05e
CLRAN3	=	$c05f

*--- The firmware routines

AUXMOVE	=	$C311
INIT	=	$FB2F
TABV	=	$FB5B
BS	=	$FC10
HOME	=	$FC58
CLREOL	=	$FC9C
WAIT	=	$FCA8
RDKEY	=	$FD0C
KEYIN	=	$FD1B
GETLNZ	=	$FD67
GETLN	=	$FD6A
CROUT	=	$FD8E
PRBYTE	=	$FDDA
COUT	=	$FDED
IDROUTINE	=	$FE1F
SETNORM	=	$FE84
SETKBD	=	$FE89
BEEP	=	$FF3A

*------------------------------------------------------------------------------
*
* LES POINTS D'ENTREE
*
*------------------------------------------------------------------------------

	ldx	#0	; relocate the loader
]lp	lda	$2000,x
	sta	$bc00,x
	lda	$2000+$100,x
	sta	$bc00+$100,x
	lda	$2000+$200,x
	sta	$bc00+$200,x
	inx
	bne	]lp
	
	jmp	leLOADER
	
*------------------------------------------------------------------------------
* LE CODE PRINCIPAL
*------------------------------------------------------------------------------

leLOADER
	jsr	PRODOS	; get the prefix
	dfb	$c7
	da	proGETPFX

	jsr	PRODOS	; set it
	dfb	$c6
	da	proGETPFX
	
	jsr	loadALL	; on charge tout
	bcs	leQUITTER	; mais on quitte sur erreur
	
	jmp	leJEU	; on saute en $800

leQUITTER	jsr	PRODOS	; exit
	dfb	$65
	da	proQUIT
	brk	$bd	; on ne se refait pas ;-)


*------------------------------------------------------------------------------
*
* LES ROUTINES DU LOADER
*
*------------------------------------------------------------------------------

*
*--- Le fichier ENGINE
*

loadALL	jsr	openFILE	; on ouvre le fichier engine
	bcc	loadALL1
loadALLERR	sec
	rts
loadALL1	lda	proGETEOF+4	; le fichier Engine
	bne	loadALLERR	; ne doit pas dépasser $1800 bytes
	lda	proGETEOF+3
	cmp	#$18
	bcc	loadALL2
	lda	proGETEOF+2	; $18xx ou pas ?
	bne	loadALLERR
loadALL2	jsr	readFILE	; on lit le fichier engine
	bcs	loadALLERR
loadALL3	jsr	closeFILE	; on ferme le fichier

*
*--- Prepare the installation of the game data for the engine
*

	jsr	getHOOKTABLE	; get our table from the engine please
	sty	dpTO	
	stx	dpTO+1

	ldy	#2	; get dataVERSION address
	lda	(dpTO),y	; and set it to dpTO
	tax
	iny
	lda	(dpTO),y
	sta	dpTO+1
	stx	dpTO
	
*
*--- Le fichier GAMEDATA
*

	lda	#<pGAMEDATA	; pointer to filename
	sta	proOPEN+1
	lda	#>pGAMEDATA
	sta	proOPEN+2
	
	lda	#<dataVERSION	; read buffer
	sta	proREAD+2
	lda	#>dataVERSION
	sta	proREAD+3

	lda	#$00	; read $100 bytes
	sta	proREAD+4
	lda	#$01
	sta	proREAD+5

	jsr	openFILE	; on ouvre le fichier game data
	bcc	loadDATA1
loadDATAERR	rts
loadDATA1	jsr	readFILE	; read $100 bytes
	bcs	loadDATAERR

*
*--- We are at the table of pointers
*

	ldx	dataVERSION	; take the offset to tblGame
	ldy	#0
]lp	lda	dataVERSION,x
	sta	dataVERSION,y
	iny
	inx
	bne	]lp
	
*
*--- Check gamedata
*

	lda	dataVERSION	; does the loader support
	cmp	#engineVERSION	; the game data version?
	bne	loadDATAERR	; no, sorry!

	lda	dataPICTURES	; now get the type of pictures
	and	#%01111111	; v1.3 - mask bit 7 for no pics
	cmp	#picUNSUPPORTED
	bcs	loadDATAERR	; unless it is unsupported
	tax
	lda	tblLOAD,x
	sta	proREAD+3	; the load address

	lda	#>loadBUFFER	; now calculate the max length we can load
	sec
	sbc	proREAD+3
	sta	proREAD+5	; and save it

	lda	proGETEOF+4	; le fichier Gamedata
	bne	loadDATAERR	; ne doit pas dépasser proREAD+5 bytes
	lda	proGETEOF+3
	cmp	proREAD+5
	bcc	loadDATA2
	lda	proGETEOF+2	; LLxx ou pas ?
	bne	loadDATAERR

loadDATA2	jsr	PRODOS	; move to the beginning of the file
	dfb	$ce
	da	proSETMARK
	bcs	loadDATAERR

	jsr	readFILE	; load the gamedata engine
	bcs	loadDATAERR
	jsr	closeFILE	; close the gamedata engine

*
*--- Init the game
*

	lda	proREAD+2	; save the gamedata address
	sta	dpDATA
	lda	proREAD+3
	sta	dpDATA+1
	lda	#0
	sta	dpDATA+2
	jsr	initGAME	; init the game data
	jsr	initTEXT	; init the text of the game

	lda	dataPICTURES	; v1.3 - no pics to display if there are no pics!
	bmi	loadDATA3
	
	lda	#0	; load the title screen
	jsr	chargeIMAGEALT

* Olivier wants a time loop here instead

	lda	#true	; display it
	jsr	switchVIDEOTER

	ldy	#20
]lp	lda	KBD
	bmi	loadDATA3
	lda	#0
	jsr	WAIT
	dey
	bne	]lp

loadDATA3	sta	KBDSTROBE	
	clc
	rts		; return and say OK
	
*--- Data

pENGINE	str	'DATA/ENGINE'
pGAMEDATA	str	'DATA/GAMEDATA'

tblLOAD	hex	20	; no pics
	hex	24	; 40-col
	hex	28	; 80-col
	hex	24	; GR
	hex	28	; DGR
	hex	40	; HGR
	hex	40	; HGR mono
	hex	40	; DHGR
	hex	40	; DHGR mono
	hex	40	; SHR
	hex	40	; SHR
	
*--------------------------------------
* Tout ce qui est ProDOS
*--------------------------------------

openFILE	jsr	PRODOS
	dfb	$c8
	da	proOPEN
	bcc	openFILE1
	sta	proERR
	rts

openFILE1	lda	proOPEN+5	; zou, on prend l'ID
	sta	proGETEOF+1
	sta	proSETMARK+1
	sta	proREAD+1
	sta	proCLOSE+1

	jsr	PRODOS	; longueur du fichier
	dfb	$d1
	da	proGETEOF
	sta	proERR
	rts

*
*---
*
	
readFILE	jsr	PRODOS	; lecture du fichier
	dfb	$ca
	da	proREAD
	sta	proERR
	rts

*
*---
*

closeFILE	jsr	PRODOS	; fermeture du fichier
	dfb	$cc
	da	proCLOSE
	sta	proERR
	rts

*--- Data

proERR	ds	1	; erreur ProDOS rencontrée

proQUIT	dfb	$4
	ds	1
	ds	2
	ds	1
	ds	2	

proGETPFX	dfb	$1
	da	ptrPREFIX

proOPEN	dfb	$3
	da	pENGINE	; pathname (par défaut, le moteur)
	da	proBUFFER	; io_buffer
	ds	1	; ref_num

proREAD	dfb	$4
	ds	1	; ref_num
	da	leJEU	; data_buffer
	dw	$1800	; request_count
	ds	2	; transfer_count

proCLOSE	dfb	$1
	ds	1	; ref_num

proSETMARK	dfb	$2
	ds	1	; ref_num
	ds	3	; position ($000000)
	
proGETEOF	dfb	$2
	ds	1	; ref_num
	ds	3	; eof

	ds	\
	
*--- Données par défaut

dataVERSION		; 0 - minimum version required 1.0 here
	dfb	engineVERSION
dataMEMORY		; 1 - the game runs fine on a 64K machine
	dfb	data64K
dataTEXT		; 2 - text is in 80-col
	dfb	text80COL
dataPICTURES		; 3 - pictures are HGR monochrome
	dfb	picHGRMONO
picCOMPRESSION		; 4 - no compression
	dfb	compNONE
lettresOK		; 5 - définit les lettres pour dire oui pour les messages du système
	asc	"Oo"
tailleSYSTEMDICTIONNAIRE	; 6 - nombre de mots dans le dictionnaire systeme
	dfb	16+1
tailleDICTIONNAIRE	; 7 - nombre de mots de dans le dictionnaire du jeu
	dfb	100+1
nbLIEUX		; 8 - nombre de décors dans le jeu
	dfb	20
nbPIECES		; 9 - nombre de pièces dans le jeu
	dfb	12
imageTITRE		; 10 - numéro de l'image titre
	dfb	12
messageTITRE		; 11 - message de titre
	dfb	74
dialogueTITRE		; 12 - message de titre
	dfb	1
nbCONDITIONS		; 13 - nombre de conditions pour gagner
	dfb	22
deathCONDITION		; 14 - condition à vérifier qu'on ne voit pas mort !
	dfb	28
conditions		; 15 - le tableau des conditions du jeu (par défaut, tout est à faux (=0))
	ds	sizeCONDITIONS

	ds	\
	